#!/usr/bin/env python
###################################
#
# Requires airsmod binary to work, set location in AIRS_MODIS_BIN
#
###################################

import logging
log = logging.getLogger(__name__)

import subprocess

from pyhdf.SD import *  #FIXME: don't import *
from intercal.hs2modis import Airs2Modis

import numpy as np
import numpy.ma as ma

import time
timer = time.time

from config import *


def airs2modis_translate(airsfile, nchan):
  sd = SD(airsfile)
  rads = sd.select('radiances')[:]
  a2m = Airs2Modis(L2_PROPERTIES, SRF_ANCIL)
  rad, bt = a2m(nchan, rads)
  return rad, bt


def airs2modis_transform(airsfile, modisfile):
  amfile = "am_transform.hdf"
  # FIXME: LD_LIBRARY_PATH FOR RUNNING ON STORM, CHANGE BEFORE RUNNING ANYWHERE ELSE

  t0 = timer()

  cmd = "%s %s %s %s" % (AIRS_MODIS_BIN, airsfile, modisfile, amfile)
  log.debug(cmd)
  subprocess.check_call(cmd, shell=True)

  # Grab the important bits out of the created file
  cv = SD(amfile, SDC.READ)
  xTracks = cv.select('MODIS_Across_Track_IDX')[:]   # max ...?
  yTracks = cv.select('MODIS_Along_Track_IDX')[:]    # max ...?
  nfovs = cv.select('Number_Of_MODIS_FOV')[:]
  cv.end()

  m = SD(modisfile)
  nscans = getattr(m, "Number of Scans")
  m.end()

  # Get rid of the airsmod file, we don't need it anymore
  cmd = "rm %s" % (amfile)
  log.debug(cmd)
  subprocess.check_call(cmd, shell=True)

  t1 = timer()
  log.info("airsmod step took: %s seconds" % (str(t1 - t0)))

  # FIXME: MOVE THESE ONCE WE KNOW WHAT WE THINK OF NUMBA!

  t0 = timer()
  mapping = airs2modis_create_mapping(nscans, xTracks, yTracks, nfovs)
  t1 = timer()

  log.info("airs2modis_create_mapping took: %s seconds" % (str(t1 - t0)))

  return mapping


def airs2modis_create_mapping(nscans, xTracks, yTracks, nfovs):
  mapping = np.empty((nscans * 10, 1354, 2), dtype=np.int) 
  mapping.fill(-1) 

  scanlines, scanangles = nfovs.shape

  for s in range(scanlines):
    for a in range(scanangles):

        nfov = nfovs[s][a]
        if nfov == 0:
          continue

        xs = xTracks[s][a]
        ys = yTracks[s][a]

        idxs = np.column_stack((ys, xs))
        for fov in range(nfov):
          yx = idxs[fov]
          y = yx[0] - 1
          x = yx[1] - 1

          mapping[y][x][0] = s
          mapping[y][x][1] = a

  mmapping = ma.masked_equal(mapping, -1)
  ma.set_fill_value(mmapping, 0)
  return mmapping


def apply_mapping(mapping, data):
  datamapped = data[mapping[:,:,0], mapping[:,:,1]]
#  datamasked = ma.array(datamapped, mask=mapping.mask[:,:,0], fill_value=0.0)
  datamasked = ma.array(datamapped, mask=mapping.mask[:,:,0], fill_value=-999.0)
  return datamasked.filled()


# CALL FOR TESTING: a2m.py <airs_file> <modis_file>
if __name__ == "__main__":
  import sys
  print sys.argv
  _, airsfile, m03 = sys.argv

  print " --- CALLING airs2modis TRANSFORM"
  mapping = airs2modis_transform(airsfile, m03)

  for i in range(31,36):
    print " --- CALLING airs2MODIS TRANSLATE for band %d" % (i)
    rad, bt = airs2modis_translate(airsfile, i)

    vrad = apply_mapping(mapping, rad)
    vbt = apply_mapping(mapping, bt)

#    import pdb; pdb.set_trace()

    def plot(arr, outname):
      import matplotlib.pyplot as plt
      plt.imshow(arr)
      plt.gray()
      plt.colorbar()
      plt.show()
      plt.savefig(outname)
      plt.clf()

    plot(vrad, "rad%d.png" % (i))
    plot(vbt, "bt%d.png" % (i))

#    import pdb; pdb.set_trace()


