def planck_wl(wl, tb):
    """ Planck func: get radiances from brightness temps at given wavelength

    Lazy validation here: http://ncc.nesdis.noaa.gov/data/planck.html

    Args:
        | wl: scalar wavelength in [um]
        | tb: 2d brightness temperature array in [K]

    Returns:
        | 2d array of spectral radiances in W/(m^2*um*sr)
    """
    import numpy as np
    c = 2.998e8 # [m/s] speed of light
    h = 6.626e-34 # [J*s] Planck's constant
    kb = 1.381e-23 # [J/K] Boltzmann's constant

    wl_m = wl * 1.0e-6 # wavelength in [m]

    b_t = (2.0*h*c**2.0) / (wl_m**5.0 * (np.exp((h*c)/(kb*wl_m*tb)) - 1))

    return b_t * 1.0e-6 # convert to W/(m^2*um*sr)
