#!/usr/bin/env python
# encoding: utf-8
"""
Author: Nick Bearson, nickb@ssec.wisc.edu
Copyright (c) 2012 University of Wisconsin SSEC. All rights reserved.
"""

from descriptors import reify
from config import *

class Sensor(object):
  """
  This is a basic stub class for all of the functions a sensor should provide.  They are just a simplistic template, 
  and return NotImplementedError exceptions in case a subclass should forget to implement them.

  Takes a filename
  """
	
  def __init__(self, file_list, *args, **kwargs):
    self._file_list = file_list
  
  @reify
  def ReflectiveSolarBands():
    raise NotImplementedError()
  
  @reify
  def ReflectiveBandNums():
    raise NotImplementedError()

  @reify
  def ReflectiveBandCenters():
    raise NotImplementedError()
  
  @reify
  def EmissiveBands():
  	raise NotImplementedError()
  
  @reify
  def EmissiveBandNums():
    raise NotImplementedError()
  
  @reify
  def EmissiveBandCenters():
    raise NotImplementedError()

  @reify
  def BrightnessTemperatureBands():
    raise NotImplementedError()
  
  @reify
  def BrightnessTemperatureBandNums():
    raise NotImplementedError()
  
  @reify
  def BrightnessTemperatureBandCenters():
    raise NotImplementedError()
  
  @reify
  def Height():
  	raise NotImplementedError()
  
  
  @reify
  def SensorZenith():
  	raise NotImplementedError()
  
  
  @reify
  def SensorAzimuth():
  	raise NotImplementedError()
  
  
  @reify
  def Range():
  	raise NotImplementedError()
  
  
  @reify
  def SolarZenith():
  	raise NotImplementedError()
  
  
  @reify
  def SolarAzimuth():
  	raise NotImplementedError()
  
  
  @reify
  def Latitude():
  	raise NotImplementedError()
  
    
  @reify
  def Longitude():
  	raise NotImplementedError()
  
  
  @reify
  def ScanStartTime():
  	raise NotImplementedError()
  
  
  @reify
  def LandSeaMask(self):
    """
    NOTE: modis uses 500m to retrieve 6 points within the 1km footprint; we don't have an equivalent for viirs at the moment
          so we use a 3x3 roaming box to do a poor man's version that can be applied to all sensors
    """
    def poormansBlur(lsm):
      xs, ys = lsm.shape
      newlsm = lsm.copy()
      for x in range(xs):
        for y in range(ys):
          if x == 0 or x == xs-1 or y == 0 or y == ys-1:
             continue
          sample = lsm[x-1:x+2,y-1:y+2]  # get 3x3 roaming box
          newlsm[x][y] = aggregate_lw(sample)
      return newlsm
    
    """
    FIXME: doing this in horrible c-like python; need to verify correctness before rewrite

    reference: PGE01/MOD_PR03/GEO_aggregate.c
    """
    def aggregate_lw(sample):
      import numpy as np
      SHALLOW_OCEAN, DRYLAND, COAST, SHALLOW_INLAND, EPHEMERAL, DEEP_INLAND, CONTINENTAL, DEEP_OCEAN = range(8)
      counts = np.bincount(sample.flatten(), minlength=8)

      land = counts[DRYLAND] + counts[COAST]

      inland = counts[SHALLOW_INLAND] + counts[DEEP_INLAND]
      ocean = counts[SHALLOW_OCEAN] + counts[CONTINENTAL] + counts[DEEP_OCEAN]
      moderate = counts[DEEP_INLAND] + counts[CONTINENTAL]
      water = inland + ocean + counts[EPHEMERAL]

      lw_sum = land + water

      if counts[EPHEMERAL] > (lw_sum / 2):
        return EPHEMERAL
      if (land + counts[EPHEMERAL]) > (lw_sum / 2):
        if water > 0:
          return COAST
        else:
          return DRYLAND
      else:
        shallow = counts[EPHEMERAL] + counts[SHALLOW_INLAND] + counts[SHALLOW_OCEAN]
        if shallow > (water / 2):
          if inland > (water / 2):
            return SHALLOW_INLAND
          else:
            return SHALLOW_OCEAN

        if inland > (water / 2):
          return DEEP_INLAND

        shallow += moderate  # C:  /* Redefine "shallow" downwards */
        if shallow > (water / 2):
          return CONTINENTAL
        else:
          return DEEP_OCEAN

    from demlw import DemLW
    demlw = DemLW(DEMLW_DIR)
    lw = demlw.latlon2demlw(self.Longitude, self.Latitude)
    blurredlw = poormansBlur(lw)
    return blurredlw
  
  
